#ifndef __STR_FIND_H
#include "str_find.h"
#endif
/*********************************************************************
 *
 * STR_FIND.C --     ,   
 *  .
 *
 * !       MS-DOS.
 *  ,     Windows,  
 *   .
 *
*********************************************************************/

#ifndef __STRING_H
#include <string.h>
#endif

/* #ifndef __ALLOC_H
#include <alloc.h>
#endif */

#ifndef __STDLIB_H
#include <stdlib.h>
#endif

/******************26/10/06 13:15************************************
 *
 *  isvalid_str
 *
 * 	      
 *	      .
 *
 * :
 *	*str 	--  
 *	n 	--   
 *	*il1 	--   
 *
 * :
 *
 *	     
 *	 (*il1 > n),  :
 *	E_BAD_STR.   E_OK.
 *
 *  :
 *
 *	    STRING.H
 *
********************************************************************/
int isvalid_str( UCHAR *str, int n, int *il1 )
{
  *il1 = strlen( (const char *)str );
  if ( *il1 > n ) return E_BAD_STR;
  return E_OK;
}

/******************26/10/06 13:18************************************
 *
 *  isvalid_strs
 *
 *	       ,
 *	       
 *	    .
 *
 * :
 *	*str 		--  
 *	*substr 	--  
 *	num 		--    
 *			   (    )
 *	n 		--   
 *	*il1 		--   
 *	*il2 		--   
 *	*n1 		--      (string + substring)
 *
 * :
 *
 *	1.       
 *	    .
 *	2.      ,
 *	    : E_BAD_STR.
 *	3.     ,
 *	    : E_BAD_SUBSTR.
 *	4.      ,
 *	    : E_BAD_NUM.
 *	5.    ,   
 *	        : E_OK.
 *
 *  :
 *
 *	: isvalid_str   .
 *
********************************************************************/

int isvalid_strs( UCHAR *str, UCHAR *substr, int num, int n, int *il1, int *il2, int *n1 )
{
  if( isvalid_str( str, n, il1 ) != E_OK) return E_BAD_STR;
  if( isvalid_str( substr, n, il2 ) != E_OK) return E_BAD_SUBSTR;
  if( num > n ) return E_BAD_NUM;
  *n1 = *il1 + *il2;
  return E_OK;
}

/******************26/10/06 13:28************************************
 *
 *  istrFind_FWD
 *
 *	       "",
 *	 .
 *
 * :
 *	*str 		-- ,    .
 *	*substr 	-- ,    .
 *	num 		--    ,
 *			      .
 *	n 		--   .
 *	*il2 		--  .
 *
 * :
 *
 *	1.       .
 *	2.     .
 *	3.        ,
 *	       .
 *	4.   ,     .
 *	5.    E_NO_OCCURENCE
 *	   ( ).
 *
 *  :
 *	: isvalid_strs   .
 *
********************************************************************/

int istrFind_FWD( UCHAR *str, UCHAR * substr, int num, int n, int *il2 )
{
  int il1, i, j, k, n1, ierr;
  if( (ierr=isvalid_strs( str, substr, num, n, &il1, il2, &n1 )) != E_OK) return ierr;
  if( n1 < 0 ) return E_BAD_SUBSTR;
  for ( i=num; i <= il1; i++ )
  {
    for( j=0, k=i; j < *il2, k < il1; j++, k++)
    {
	if( str[k] != substr[j] ) break;
    }
    if( j == *il2 ) return i;
  }
  return E_NO_OCCURENCE;
}


/**********************26/10/09 13:35*********************************
 *
 *  istrFind_BWD:
 *
 *	       "",
 *	 .
 *
 * :
 *	*str 	-- ,    .
 *	*substr -,    .
 *	num 	--    ,    .
 *	n 	--   .
 *	*il2 	--  .
 *
 * :
 *
 *	1.       .
 *	2.     .
 *	3.        ,
 *	       .
 *	4.   ,     .
 *	5.    E_NO_OCCURENCE
 * 	   ( ).
 *
 *  :
 *
 *	: isvalid_strs   .
 *
*********************************************************************/

int istrFind_BWD( UCHAR *str, UCHAR *substr, int num, int n, int *il2 )
{
  int il1, i, j, k, n1, ierr;
  if( (ierr=isvalid_strs( str, substr, num, n, &il1, il2, &n1 )) != E_OK) return ierr;
  n1 = num - *il2;
  if( n1 < 0 ) return E_BAD_SUBSTR;
  for ( i=n1; i>=0; i-- )
  {
    for( j=0, k=i; j < *il2, k < il1; j++, k++)
    {
	if( str[k] != substr[j] ) break;
    }
    if( j == *il2 ) return i;
  }
  return E_NO_OCCURENCE;
}

/***********************************************************************
 *
 *  istrDel
 *
 *	      ,   
 *	ipos   il0      
 *	*newstr.
 *
 *` :
 *	*str 	--  ;
 * 	ipos 	--    ;
 *	il0 	--   ;
 *	n 	--     (  ).
 *	*newstr -,     .
 *
 *  :
 *
 *	E_OK 		--   .
 *	E_BAD_STR	--     (  ).
 *	E_BAD_NUM	--        .
 *
 *
 *  :
 *
 *	: isvalid_str   .
 *	      STRING.H
 *
*************************************************************************/

int istrDel( UCHAR *str, int ipos, int il0, int n, UCHAR *newstr )
{
  int il1;
  if( isvalid_str( str, n, &il1 ) != E_OK) return E_BAD_STR;
  if( ipos > n || il0 > n ) return E_BAD_NUM;
  strncpy( (char *)newstr, (const char *)str, (size_t)ipos );
  strncat( (char *)newstr, (const char *)&str[ipos+il0], (size_t)n );
  return E_OK;
}


/***********************************************************************
 *
 *  istrIns
 *
 *	      *str   
 *	*substr,    num     
 *	*newstr.
 *
 *` :
 *	*str 	--  ;
 *	*substr - ;
 * 	num 	--    ;
 *	n 	--     (  ).
 *	*newstr -,     .
 *
 *  :
 *
 *	    isvalid_strs();
 *
 *	E_OK 		--   .
 *	E_BAD_SUBSTR -    
 *			   (    ).
 *
 *  :
 *
 *	: isvalid_strs   .
 *	      STRING.H
 *
 *
 *
*************************************************************************/

int istrIns( UCHAR *str, UCHAR *substr, int num, int n, UCHAR *newstr )
{
  int il1, il2, n1, ierr;
  if( (ierr=isvalid_strs( str, substr, num, n, &il1, &il2, &n1 )) != E_OK) return ierr;
  if( n1 > n ) return E_BAD_SUBSTR;
  strncpy( (char *)newstr, (const char *)str, (size_t)num );
  strncat( (char *)newstr, (const char *)substr, (size_t)il2 );
  strncat( (char *)newstr, (const char *)&str[num], (size_t)n1 );
  return 0;
}

/***********************************************************************
 *
 *  istrRepl
 *
 *	 istrRepl      *str,
 *	   ipos     il0,
 *	   *substr     
 *	*newstr.
 *
 *` :
 *	*str 	--  ;
 *	ipos	--    ,   
 *		       *substr;
 *	il0	--      
 *		   (     *substr)
 *	*substr --  ;
 *	*newstr -- ,     .
 *
 *  :
 *
 *	    isvalid_strs;
 *	    istrDel;
 *	    istrIns;
 *
 *	E_OK 		--   .
 *	E_BAD_SUBSTR 	--    
 *			   (  
 *			     ).
 *	E_BAD_NUM	--      
 *			      .
 *	E_BAD_MEMORY_HEAP -   
 *			       .
 *
 *  :
 *
 *	: isvalid_strs   .
 *	: istrDel   .
 *	: istrIns   .
 * 	      ALLOC.H
 *
*************************************************************************/

int istrRepl( UCHAR *str, int ipos, int il0, UCHAR *substr, int n, UCHAR *newstr )
{
  int il1, il2, n1, ierr;
  UCHAR *temp = NULL;
  if( ipos > n || il0 > n ) return E_BAD_NUM;
  if( (ierr=isvalid_strs( str, substr, ipos, n, &il1, &il2, &n1 )) != E_OK) return ierr;
  if( il1-(il0-ipos)+il2 > n ) return E_BAD_SUBSTR;
  if( (temp = (UCHAR *) calloc( n, sizeof(UCHAR)) ) == NULL)
    return E_BAD_MEMORY_HEAP;
  ierr=istrDel( str, ipos, il0, n, temp );
  if( ierr != E_OK ) return ierr;
  ierr=istrIns( temp, substr, ipos, n, newstr );
  free ( temp );
  if( ierr != E_OK ) return ierr;
  return E_OK;
}

/***********************************************************************
 *
 *  istrDelete
 *
 *	      ,   
 *      ipos   il0      ne  
 *
 *` :
 *	*str 	--  ;
 * 	ipos 	--    ;
 *	il0 	--   ;
 *	n 	--     (  ).
 *	*newstr -,     .
 *
 *  :
 *
 *	E_OK 		--   .
 *	E_BAD_STR	--     (  ).
 *	E_BAD_NUM	--        .
 *      E_BAD_MEMORY_HEAP -        ""
 *
 *
 *  :
 *
 *      : istrDel   .
 *	      STRING.H
 *            ALLOC.H
 *
*************************************************************************/

int istrDelete( UCHAR *str, int ipos, int il0, int n )
{
  int ierr = E_OK;
  UCHAR *temp = NULL;
  if( (temp = (UCHAR *) calloc (n, sizeof( UCHAR ))) == NULL )
    return E_BAD_MEMORY_HEAP;
  ierr=istrDel( str, ipos, il0, n, temp );
  if( ierr != E_OK ) return ierr;
  strncpy( (char *)&str[0], (const char *)&temp[0], (size_t)n);
  free( temp );
  return E_OK;
}

/***********************************************************************
 *
 *  istrInsert
 *
 *	      *str   
 *      *substr,    num     
 *      *str.
 *
 *` :
 *	*str 	--  ;
 *	*substr - ;
 * 	num 	--    ;
 *	n 	--     (  ).
 *
 *  :
 *
 *          istrIns;
 *
 *      E_BAD_MEMORY_HEAP -        ""
 *
 *  :
 *
 *      : istrIns   .
 *	      STRING.H
 *            ALLOC.H
 *
 *
*************************************************************************/

int istrInsert( UCHAR *str, UCHAR *substr, int num, int n )
{
  int ierr = E_OK;
  UCHAR *temp = NULL;
  if( (temp = (UCHAR *) calloc (n, sizeof( UCHAR ))) == NULL )
    return E_BAD_MEMORY_HEAP;
  ierr=istrIns( str, substr, num, n, temp );
  if( ierr != E_OK ) return ierr;
  strncpy( (char *)str, (const char *)temp, (size_t)n);
  free( temp );
  return E_OK;
}

/***********************************************************************
 *
 *  istrReplace
 *
 *	 istrRepl      *str,
 *	   ipos     il0,
 *         *substr
 *        .
 *
 *` :
 *	*str 	--  ;
 *	ipos	--    ,   
 *		       *substr;
 *	il0	--      
 *		   (     *substr)
 *	*substr --  ;
 *
 *  :
 *
 *          istrRepl;
 *
 *	E_OK 		--   .
 *	E_BAD_SUBSTR 	--    
 *			   (  
 *			     ).
 *	E_BAD_NUM	--      
 *			      .
 *	E_BAD_MEMORY_HEAP -   
 *			       .
 *
 *  :
 *
 *      : istrRepl   .
 *	      STRING.H
 * 	      ALLOC.H
 *
*************************************************************************/

int istrReplace( UCHAR *str, int ipos, int il0, UCHAR *substr, int n )
{
  int ierr = E_OK;
  UCHAR *temp = NULL;
  if( (temp = (UCHAR *) calloc (n, sizeof( UCHAR ))) == NULL )
    return E_BAD_MEMORY_HEAP;
  ierr = istrRepl( str, ipos, il0, substr, n, temp );
  if( ierr != E_OK ) return ierr;
  strncpy( (char *)&str[0], (const char *)&temp[0], (size_t)n);
  free( temp );
  return E_OK;
}
