/********************************************************************
 *
 *      Inprise/Borland C++.
 *
********************************************************************/

#define STRICT

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifndef __RUSENG_H
#include "ruseng.h"
#endif

#ifndef __CTYPE_H
#include <ctype.h>
#endif

/*       */

/* The Simple Program for checking a Symbol on Validation*/
/**********************************************************
 *
 * Function is_valid_symbol
 *
 * The Function is_valid_symbol checks if the symol of
 * a string str with index index may be alphabetical
 * symbol.
 *
 * FUNCTION PARAMETERS:
 *
 * UCHAR str is a buffer from which takes the symbol
 * int * index is a pointer of a symbol index in a buffer
 * str
 *
 * FUNCTION RETURNS:
 *
 * The current symbol if symbol is valid
 * 0x00 otherwice
 *
 * This function has a modifier
 *
**********************************************************/

UCHAR is_valid_symbol( int index, UCHAR *str )
{
  UCHAR cSymbol;
  cSymbol = str[index];
  if( (cSymbol >= 'A') || (cSymbol == '-') )  /* Find the Valid Letter */
  {
      return cSymbol;
  }
  else
      return 0x00;
}


/*      */

/* Program for checking a Symbol on validation */

/**********************************************************
 *
 * Function is_valid_alphabet
 *
 * Function is_valid_alphabet checks a symbol in a position
 * index of string str on validation to the language
 * ilanguade and the Code Page iCode_Tab
 *
 * Parameters of the function:
 *
 * *index       Pointer to the current symbol position in the string
 * *str         Pointer to the string which contains checking symbol
 * ilanguage    The Current Language (Russian, English or Both).
 * iCode_Tab    The Current Code Page (for Russian and both Russian/English Languages
 *
 * Valid Languages:   Russian, English
 * Valid Code pages
 * for Russian Language:  866. 1251. KOI-8R
 * Function Returns:
 *
 * The Current Symbol   if Symbol is valid
 * 0x00                 Otherwise
 *
 * Function has modificator
 *
**********************************************************/

UCHAR is_valid_alphabet( int index, UCHAR *str, int ilanguage, int iCode_Tab)
{
/*  : ,  */

/*  : : ASCII ; : DOS-866, Windows-1251; KOI-8R */

  register UCHAR cSymbol;

switch( ilanguage )
{

case LANG_ENGLISH:

  if( cis_engalphabet( str[index] ) )        /* Find latin letters */
	{
		cSymbol = cis_engalphabet( str[index]);
		return cSymbol;
	}

	return 0x00;

case LANG_RUS:

  if( cis_rusalphabet( str[index], iCode_Tab ) )         /* Find Rus Symbols */
	{
		cSymbol = cis_rusalphabet( str[index], iCode_Tab );
		return cSymbol;
	}

	return 0x00;

default:

  /* Russian and English languages */

  if( cis_engalphabet( str[index] ) )                    /* Find English symbols */
	{
		cSymbol = cis_engalphabet( str[index]);
		return cSymbol;
	}

  if( cis_rusalphabet( str[index], iCode_Tab ) )         /* Find the Cyrillic
                                                           symbol in specified code page */
	{
		cSymbol = cis_rusalphabet( str[index], iCode_Tab );
		return cSymbol;
	}

	return 0x00;

}

}

/*      */

/**********************************************************
 *
 * Function cis_rusalphabet
 *
 * This function checks the symbol cSymbol for validation
 * to Cyrillic alphabet in Code Page Code_Tab.
 *
 * Valid Language is Russian
 * Valid Code Pages:  CP-866, CP-1251, KOI8-R
 *
 * Parameters of the Function:
 *
 * cSymbol is tested symbol
 * Code_Tab is valid code page.
 *
 * Function returns:
 *
 * cSymbol if this symbol is Russian
 * 0x00 otherwise
 *
 * Function has modifier
 *
**********************************************************/

UCHAR cis_rusalphabet( UCHAR cSymbol, int Code_Tab )
{

switch( Code_Tab )				// Choose a Russian Code Table

{
	case CODETAB_866:			// Alternative DOS Code Table

	if ((cSymbol >= 0x80 && cSymbol <= 0xAF)
		|| (cSymbol >= 0xE0 && cSymbol <= 0xF1) )

		return cSymbol;

	else

		return 0x00;

	case CODETAB_1251:		// Windows Code Table

	if ((cSymbol >= 0xC0 && cSymbol <= 0xFF)
		|| (cSymbol == 0xB8 || cSymbol == 0xA8))

		return cSymbol;

	else

		return 0x00;

	case CODETAB_KOI:		// KOI-8 Code Table

	if (cSymbol >= 0xC0 && cSymbol <= 0xFE)

		return cSymbol;

	else

		return 0x00;

}

return 0x00;

}

/*      */

/**********************************************************
 *
 * Function cis_engalphabet
 *
 * This function checks the symbol cSymbol for validation
 * to Latin alphabet.
 *
 * Valid Language is Ennglish
 * Valid Code Pages:  Not Checked
 *
 * Parameters of the Function:
 *
 * cSymbol is tested symbol
 *
 * Function returns:
 *
 * cSymbol if this symbol is English
 * 0x00 otherwise
 *
 * Function has modifier
 *
**********************************************************/

UCHAR cis_engalphabet( UCHAR cSymbol )
{



	if ((cSymbol >= 0x41 && cSymbol <= 0x5A)
		|| (cSymbol >= 0x61 && cSymbol <= 0x7A) )

		return cSymbol;

	else

		return 0x00;

}

/*        */

/**********************************************************
 *
 * Function cto_rusupper
 *
 * This function converts the symbol cSymbol to upper case
 * in Cyrillic alphabet in Code Page Code_Tab.
 *
 * Valid Language is Russian
 * Valid Code Pages:  CP-866, CP-1251, KOI8-R
 *
 * Parameters of the Function:
 *
 * cSymbol is current symbol
 * Code_Tab is valid code page.
 *
 * Function returns:
 *
 * cSymbol if this symbol can't convert to upper case or
 * symbol converted to upper case.
 *
 * Function has modifier
 *
**********************************************************/

UCHAR cto_rusupper( UCHAR cSymbol, int Code_Tab )
{

switch( Code_Tab )        /* Choose a Russian Code Table */

{
  case CODETAB_866:     /* Alternative DOS Code Table */

	if (cSymbol >= 0xA0 && cSymbol <= 0xAF)
	return cSymbol -= 0x20;
   if (cSymbol >= 0xE0 && cSymbol <= 0xEF)
	return cSymbol -= 0x50;
   if (cSymbol == 0xF1 )
	return 0xF0;

		return cSymbol;

	case CODETAB_1251:		// Windows Code Table

   if ( cSymbol == 0xB8 )
	return 0xA8;
	if (cSymbol >= 0xE0 && cSymbol <= 0xFF)
	return cSymbol -= 0x20;

		return cSymbol;

	case CODETAB_KOI:		// KOI-8 Code Table

	if (cSymbol >= 0xC0 && cSymbol <= 0xDE)
	return cSymbol += 0x20;

		return cSymbol;

}

return cSymbol;

}

/*       */

/**********************************************************
 *
 * Function cto_ruslower
 *
 * This function converts the symbol cSymbol to lower case
 * in Cyrillic alphabet in Code Page Code_Tab.
 *
 * Valid Language is Russian
 * Valid Code Pages:  CP-866, CP-1251, KOI8-R
 *
 * Parameters of the Function:
 *
 * cSymbol is current symbol
 * Code_Tab is valid code page.
 *
 * Function returns:
 *
 * cSymbol if this symbol can't convert to lower case or
 * symbol converted to lower case.
 *
 * Function has modifier
 *
**********************************************************/

UCHAR cto_ruslower( UCHAR cSymbol, int Code_Tab )
{

switch( Code_Tab )				// Choose a Russian Code Table

{
	case CODETAB_866:			// Alternative DOS Code Table

	if (cSymbol >= 0x80 && cSymbol <= 0x8F)
	return cSymbol += 0x20;
   if (cSymbol >= 0x90 && cSymbol <= 0x9F)
	return cSymbol += 0x50;
   if (cSymbol == 0xF0 )
	return 0xF1;

		return cSymbol;

	case CODETAB_1251:		// Windows Code Table

   if ( cSymbol == 0xA8 )
	return 0xB8;
	if (cSymbol >= 0xC0 && cSymbol <= 0xDF)
	return cSymbol += 0x20;

		return cSymbol;

	case CODETAB_KOI:		// KOI-8 Code Table

	if (cSymbol >= 0xE0 && cSymbol <= 0xFE)
	return cSymbol -= 0x20;

		return cSymbol;

}

return cSymbol;

}

/*       */

/**********************************************************
 *
 * Function cis_rusupper
 *
 * This function checks the symbol cSymbol for upper case
 * in Cyrillic alphabet in Code Page Code_Tab.
 *
 * Valid Language is Russian
 * Valid Code Pages:  CP-866, CP-1251, KOI8-R
 *
 * Parameters of the Function:
 *
 * cSymbol is current symbol
 * Code_Tab is valid code page.
 *
 * Function returns:
 *
 * cSymbol if this symbol has an upper case or
 * 0x00 otherwise.
 *
 * Function has modifier
 *
**********************************************************/

UCHAR cis_rusupper( UCHAR cSymbol, int Code_Tab )
{

switch( Code_Tab )				// Choose a Russian Code Table

{
	case CODETAB_866:			// Alternative DOS Code Table

	if (cSymbol >= 0x80 && cSymbol <= 0x8F)
	return cSymbol;
   if (cSymbol >= 0x90 && cSymbol <= 0x9F)
	return cSymbol;
   if (cSymbol == 0xF0 )
	return 0xF0;

		return 0x00;

	case CODETAB_1251:		// Windows Code Table

   if ( cSymbol == 0xA8 )
	return 0xA8;
	if (cSymbol >= 0xC0 && cSymbol <= 0xDF)
	return cSymbol;

		return 0x00;

	case CODETAB_KOI:		// KOI-8 Code Table

	if (cSymbol >= 0xE0 && cSymbol <= 0xFE)
	return cSymbol;

		return 0x00;

}

return 0x00;

}

/*      . */

/**********************************************************
 *
 * Function cis_ruslower
 *
 * This function checks the symbol cSymbol for lower case
 * in Cyrillic alphabet in Code Page Code_Tab.
 *
 * Valid Language is Russian
 * Valid Code Pages:  CP-866, CP-1251, KOI8-R
 *
 * Parameters of the Function:
 *
 * cSymbol is current symbol
 * Code_Tab is valid code page.
 *
 * Function returns:
 *
 * cSymbol if this symbol has an lower case or
 * 0x00 otherwise.
 *
 * Function has modifier
 *
**********************************************************/

UCHAR cis_ruslower( UCHAR cSymbol, int Code_Tab )
{

switch( Code_Tab )				// Choose a Russian Code Table

{
	case CODETAB_866:			// Alternative DOS Code Table

	if (cSymbol >= 0xA0 && cSymbol <= 0xAF)
	return cSymbol;
   if (cSymbol >= 0xE0 && cSymbol <= 0xEF)
	return cSymbol;
   if (cSymbol == 0xF1 )
	return 0xF1;

		return 0x00;

	case CODETAB_1251:		// Windows Code Table

   if ( cSymbol == 0xB8 )
	return 0xB8;
	if (cSymbol >= 0xE0 && cSymbol <= 0xFF)
	return cSymbol;

		return 0x00;

	case CODETAB_KOI:		// KOI-8 Code Table

	if (cSymbol >= 0xC0 && cSymbol <= 0xDE)
	return cSymbol;

		return 0x00;

}

return 0x00;

}


