#define STRICT

#ifndef __STR_FIND_H
#include "str_find.h"
#endif
/*********************************************************************
 *
 * STR_FIND.C --     ,   
 *  .
 *
 *     ,   
 *    .
 *
*********************************************************************/

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifndef __STDLIB_H
#include <stdlib.h>
#endif

/******************26/10/06 13:15************************************
 *
 *  isvalid_str
 *
 * 	      
 *	      .
 *
 * :
 *	*str 	--  
 *	n 	--   
 *	*il1 	--   
 *
 * :
 *
 *	     
 *	 (*il1 > n),  :
 *	E_BAD_STR.   E_OK.
 *
 *  :
 *
 *	    STRING.H
 *
********************************************************************/
short WINAPI isvalid_str( LPCSTR str, short n, short *il1 )
{
  *il1 = lstrlen( str );
  if ( *il1 > n ) return E_BAD_STR;
  return E_OK;
}

/******************26/10/06 13:18************************************
 *
 *  isvalid_strs
 *
 *	       ,
 *	       
 *	    .
 *
 * :
 *	*str 		--  
 *	*substr 	--  
 *	num 		--    
 *			   (    )
 *	n 		--   
 *	*il1 		--   
 *	*il2 		--   
 *	*n1 		--      (string + substring)
 *
 * :
 *
 *	1.       
 *	    .
 *	2.      ,
 *	    : E_BAD_STR.
 *	3.     ,
 *	    : E_BAD_SUBSTR.
 *	4.      ,
 *	    : E_BAD_NUM.
 *	5.    ,   
 *	        : E_OK.
 *
 *  :
 *
 *	: isvalid_str   .
 *
********************************************************************/

short WINAPI isvalid_strs( LPCSTR str, LPCSTR substr, short num, short n, short *il1, short *il2, short *n1 )
{
  if( isvalid_str( str, n, il1 ) != E_OK) return E_BAD_STR;
  if( isvalid_str( substr, n, il2 ) != E_OK) return E_BAD_SUBSTR;
  if( num > n ) return E_BAD_NUM;
  *n1 = *il1 + *il2;
  return E_OK;
}

/******************26/10/06 13:28************************************
 *
 *  istrFind_FWD
 *
 *	       "",
 *	 .
 *
 * :
 *	*str 		-- ,    .
 *	*substr 	-- ,    .
 *	num 		--    ,
 *			      .
 *	n 		--   .
 *	*il2 		--  .
 *
 * :
 *
 *	1.       .
 *	2.     .
 *	3.        ,
 *	       .
 *	4.   ,     .
 *	5.    E_NO_OCCURENCE
 *	   ( ).
 *
 *  :
 *	: isvalid_strs   .
 *
********************************************************************/

short WINAPI istrFind_FWD( LPCSTR str, LPCSTR substr, short num, short n, short *il2 )
{
  short il1, i, j, k, n1, ierr;
  if( (ierr=isvalid_strs( str, substr, num, n, &il1, il2, &n1 )) != E_OK) return ierr;
  if( n1 < 0 ) return E_BAD_SUBSTR;
  for ( i=num; i <= il1; i++ )
  {
    for( j=0, k=i; j < *il2, k < il1; j++, k++)
    {
        if( str[k] != substr[j] ) break;
    }
    if( j == *il2 ) return i;
  }
  return E_NO_OCCURENCE;
}


/**********************26/10/09 13:35*********************************
 *
 *  istrFind_BWD:
 *
 *	       "",
 *	 .
 *
 * :
 *	*str 	-- ,    .
 *	*substr -,    .
 *	num 	--    ,    .
 *	n 	--   .
 *	*il2 	--  .
 *
 * :
 *
 *	1.       .
 *	2.     .
 *	3.        ,
 *	       .
 *	4.   ,     .
 *	5.    E_NO_OCCURENCE
 * 	   ( ).
 *
 *  :
 *
 *	: isvalid_strs   .
 *
*********************************************************************/

short WINAPI istrFind_BWD( LPCSTR str, LPCSTR substr, short num, short n, short *il2 )
{
  short il1, i, j, k, n1, ierr;
  if( (ierr=isvalid_strs( str, substr, num, n, &il1, il2, &n1 )) != E_OK) return ierr;
  n1 = num - *il2;
  if( n1 < 0 ) return E_BAD_SUBSTR;
  for ( i=n1; i>=0; i-- )
  {
    for( j=0, k=i; j < *il2, k < il1; j++, k++)
    {
        if( str[k] != substr[j] ) break;
    }
    if( j == *il2 ) return i;
  }
  return E_NO_OCCURENCE;
}

/***********************************************************************
 *
 *  istrDel
 *
 *	      ,   
 *	ipos   il0      
 *	*newstr.
 *
 *` :
 *	*str 	--  ;
 * 	ipos 	--    ;
 *	il0 	--   ;
 *	n 	--     (  ).
 *	*newstr -,     .
 *
 *  :
 *
 *	E_OK 		--   .
 *	E_BAD_STR	--     (  ).
 *	E_BAD_NUM	--        .
 *	E_BAD_MEMORY_HEAP -   
 *			       .
 *
 *
 *  :
 *
 *	: isvalid_str   .
 *	      STRING.H
 *
*************************************************************************/

short WINAPI istrDel( LPCSTR str, short ipos, short il0, short n, HLOCAL *hnewstr )
{
  short il1;
  LPSTR newstr;
  if( isvalid_str( str, n, &il1 ) != E_OK) return E_BAD_STR;
  if( ipos > n || il0 > n ) return E_BAD_NUM;
  newstr = (LPSTR) LocalLock( *hnewstr);
  if( newstr == NULL ) return E_BAD_MEMORY_HEAP;
  lstrcpyn( newstr, str, (int) ipos );
  lstrcat( newstr, &str[ipos+il0] );
  if( LocalUnlock( *hnewstr ) ) return E_BAD_MEMORY_HEAP;
  return E_OK;
}


/***********************************************************************
 *
 *  istrIns
 *
 *	      *str   
 *	*substr,    num     
 *	*newstr.
 *
 *` :
 *	*str 	--  ;
 *	*substr - ;
 * 	num 	--    ;
 *	n 	--     (  ).
 *	*newstr -,     .
 *
 *  :
 *
 *	    isvalid_strs();
 *
 *	E_OK 		--   .
 *	E_BAD_SUBSTR -    
 *			   (    ).
  *	E_BAD_MEMORY_HEAP -   
 *			       .
*
 *  :
 *
 *	: isvalid_strs   .
 *	      STRING.H
 *
 *
 *
*************************************************************************/

short WINAPI istrIns( LPCSTR str, LPCSTR substr, short num, short n, HLOCAL *hnewstr )
{
  short il1, il2, n1, ierr;
  LPSTR newstr;
  if( (ierr=isvalid_strs( str, substr, num, n, &il1, &il2, &n1 )) != E_OK) return ierr;
  if( n1 > n ) return E_BAD_SUBSTR;
  newstr = (LPSTR) LocalLock( *hnewstr);
  if( newstr == NULL ) return E_BAD_MEMORY_HEAP;
  lstrcpyn( newstr, str, (int) num );
  lstrcat( newstr, substr );
  lstrcat( newstr, &str[num] );
  if( LocalUnlock( *hnewstr ) ) return E_BAD_MEMORY_HEAP;
  return E_OK;
}

/***********************************************************************
 *
 *  istrRepl
 *
 *	 istrRepl      *str,
 *	   ipos     il0,
 *	   *substr     
 *	*newstr.
 *
 *` :
 *	*str 	--  ;
 *	ipos	--    ,   
 *		       *substr;
 *	il0	--      
 *		   (     *substr)
 *	*substr --  ;
 *	*newstr -- ,     .
 *
 *  :
 *
 *	    isvalid_strs;
 *	    istrDel;
 *	    istrIns;
 *
 *	E_OK 		--   .
 *	E_BAD_SUBSTR 	--    
 *			   (  
 *			     ).
 *	E_BAD_NUM	--      
 *			      .
 *	E_BAD_MEMORY_HEAP -   
 *			       .
 *
 *  :
 *
 *	: isvalid_strs   .
 *	: istrDel   .
 *	: istrIns   .
 * 	      ALLOC.H
 *
*************************************************************************/

short WINAPI istrRepl( LPCSTR str, short ipos, short il0, LPCSTR substr, short n, HLOCAL *hnewstr )
{
  short il1, il2, n1, ierr;
  HLOCAL hsztemp;
  LPCSTR sztemp;
  /*       */
  if( ipos > n || il0 > n ) return E_BAD_NUM;
  if( (ierr=isvalid_strs( str, substr, ipos, n, &il1, &il2, &n1 )) != E_OK) return ierr;
  if( il1-(il0-ipos)+il2 > n ) return E_BAD_SUBSTR;
  /*    */
  hsztemp = LocalAlloc(LHND, (UINT) n );
  if( hsztemp == NULL ) return E_BAD_MEMORY_HEAP;
  /*          */
  ierr=istrDel( str, ipos, il0, n, &hsztemp );
  if( ierr != E_OK ) 
  {
	(void) LocalFree(hsztemp );
	return ierr;
  }
  /*    */
  sztemp = (LPCSTR) LocalLock( hsztemp );
  if( sztemp == NULL )
  {
	(void) LocalFree(hsztemp );
	return E_BAD_MEMORY_HEAP;
  }
  /*       */
  ierr=istrIns( sztemp, substr, ipos, n, hnewstr );
  if( LocalUnlock( hsztemp ) ) return E_BAD_MEMORY_HEAP;
  if( LocalFree( hsztemp ) ) return E_BAD_MEMORY_HEAP;
  if( ierr != E_OK ) return ierr;
  return E_OK;
}

short WINAPI istrDelete( HLOCAL *hstr, short ipos, short il0, short n )
{
  short ierr = E_OK;
  HLOCAL htemp = NULL;
  LPCSTR temp, str;

  /*    */
  htemp = LocalAlloc(LHND, (UINT) n*sizeof( char ) );
  if( htemp == NULL ) return E_BAD_MEMORY_HEAP;
  str = (LPCSTR) LocalLock( *hstr );
  if( str == NULL )
  {
	(void) LocalFree(htemp );
	return E_BAD_MEMORY_HEAP;
  }
  
  ierr=istrDel( str, ipos, il0, n, &htemp );
  if( ierr != E_OK )
  {
	(void) LocalFree(htemp );
	return ierr;
  }
  temp = (LPCSTR) LocalLock( htemp );
  if( temp == NULL )
  {
	(void) LocalFree( htemp );
	(void) LocalUnlock( *hstr );
	return E_BAD_MEMORY_HEAP;
  }
  /*      */
  lstrcpyn( (LPSTR) str, temp, n);

  /*   */
	(void) LocalUnlock( *hstr );
	(void) LocalUnlock( htemp );
	if( LocalFree( htemp ) ) return E_BAD_MEMORY_HEAP;

  return E_OK;
}

short WINAPI istrInsert( HLOCAL *hstr, HLOCAL hsubstr, short num, short n )
{
  short ierr = E_OK;
  HLOCAL htemp;
  LPSTR temp, str;
  LPCSTR substr;

  /*    */
  htemp = LocalAlloc(LHDN, (UINT) n*sizeof( char ) );
  if( htemp == NULL ) return E_BAD_MEMORY_HEAP;
  str = (LPSTR) LocalLock( *hstr );
  substr = (LPCSTR) LocalLock( hsubstr );
  if( str == NULL || substr == NULL )
  {
	(void) LocalFree(htemp );
	(void) LocalUnlock( *hstr );
	(void) LocalUnlock( hsubstr );
	return E_BAD_MEMORY_HEAP;
  }
  
  ierr=istrIns( str, substr, num, n, &htemp );
  if( ierr != E_OK ) 
  {
	(void) LocalFree(htemp );
	(void) LocalUnlock( *hstr );
	(void) LocalUnlock( hsubstr );
	return ierr;
  }

  temp = (LPCSTR) LocalLock( htemp );
  if( temp == NULL )
  {
	(void) LocalFree(htemp );
	(void) LocalUnlock( *hstr );
	(void) LocalUnlock( hsubstr );
	return E_BAD_MEMORY_HEAP;
  }
  
  lstrcpyn( str, temp, n);

  /*   */
	(void) LocalUnlock( *hstr );
	(void) LocalUnlock( hsubstr );
	(void) LocalUnlock( htemp );
	if( LocalFree( htemp ) ) return E_BAD_MEMORY_HEAP;

  return E_OK;
}

short WINAPI istrReplace( HLOCAL *hstr, short ipos, short il0, HLOCAL hsubstr, short n )
{
  short il1, il2, n1, ierr = E_OK;
  LPSTR str;
  LPCSTR temp, substr;
  HLOCAL htemp;

  /*    */
  htemp = LocalAlloc(LHDN, (UINT) n*sizeof (char ) );
  if( htemp == NULL ) return E_BAD_MEMORY_HEAP;
  str = (LPSTR) LocalLock( *hstr );
  substr = (LPCSTR) LocalLock( hsubstr );
  if( str == NULL || substr == NULL )
  {
	(void) LocalFree(htemp );
	(void) LocalUnlock( *hstr );
	(void) LocalUnlock( hsubstr );
	return E_BAD_MEMORY_HEAP;
  }
  
  ierr = istrRepl( str, ipos, il0, substr, n, &htemp );
  if( ierr != E_OK ) 
  {
	(void) LocalFree(htemp );
	(void) LocalUnlock( *hstr );
	(void) LocalUnlock( hsubstr );
	return ierr;
  }

  temp = (LPCSTR) LocalLock( htemp );
  if( temp == NULL )
  {
	(void) LocalFree(htemp );
	(void) LocalUnlock( *hstr );
	(void) LocalUnlock( hsubstr );
	return E_BAD_MEMORY_HEAP;
  }
  
  lstrcpyn( str, temp, n);

  /*   */
	(void) LocalUnlock( *hstr );
	(void) LocalUnlock( hsubstr );
	(void) LocalUnlock( htemp );
	if( LocalFree( htemp ) ) return E_BAD_MEMORY_HEAP;

  return E_OK;
}
